from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("absolutelyright")
except PackageNotFoundError:
    __version__ = "0.0.0"

from absolutelyright._records import (
    Prompt,
    Session,
    load_sessions,
    parse_session,
    session_files,
)
from absolutelyright._stats import (
    absolutely_right,
    by_parent,
    by_project,
    corrections,
    daily_counts,
    distinctive_words,
    hour_histogram,
    images_pasted,
    model_mix,
    overview,
    phrase_children,
    phrase_counts,
    repeated_prompts,
    session_rows,
    shipped,
    slash_counts,
    tool_counts,
    weekday_histogram,
    word_counts,
)
